# encoding: utf-8

# Copyright (c) 2015, Tomas Korcak <korczis@gmail.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

require_relative '../shared'

module GoodData
  module GoodotCLI
    desc 'Manage clients in your application'
    command :clients do |c|
      c.desc 'List clients'
      c.command :list do |add|
        add.desc 'Filter on particular segment'
        add.default_value :all
        add.flag [:s, :segment]

        add.action do |global_options, options, _args|
          domain = global_options[:gd_domain]
          segment_id = options[:segment]
          table = Terminal::Table.new do |t|
            t << ['Client Id', 'Segment URI', 'Project URI']
            t << :separator
            Array(domain.segments(segment_id)).pmapcat(&:clients).each { |s| t.add_row([s.client_id, s.segment_uri, s.project_uri]) }
          end
          puts table
        end
      end

      c.desc 'Remove client'
      c.command :remove do |remove|
        remove.desc 'Client'
        remove.flag [:c, :client]

        remove.desc 'Delete the project'
        remove.default_value false
        remove.switch [:d, :'delete-project']

        remove.action do |global_options, options, _args|
          domain = global_options[:gd_domain]
          client_name = options[:client]
          fail 'Client needs to be provided' unless client_name
          client = domain.clients(client_name)
          project = client.project
          client.delete
          project.delete if options[:d]
        end
      end

      c.desc 'Reset client. Removes the project from the client.'
      c.command :reset do |reset|
        reset.desc 'Client'
        reset.flag [:c, :client]

        reset.desc 'Delete the project'
        reset.default_value false
        reset.switch [:d, :'delete-project']

        reset.action do |global_options, options, _args|
          domain = global_options[:gd_domain]
          client_name = options[:client]
          fail 'Client needs to be provided' unless client_name
          client = domain.clients.find { |cl| cl.id == client_name }
          project = client.project
          client.project = nil
          client.save
          project.delete if options[:d]
        end
      end
    end
  end
end
