# # encoding: utf-8
#
# # Copyright (c) 2015, Tomas Korcak <korczis@gmail.com>
# #
# # Permission is hereby granted, free of charge, to any person obtaining a copy
# # of this software and associated documentation files (the "Software"), to deal
# # in the Software without restriction, including without limitation the rights
# # to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# # copies of the Software, and to permit persons to whom the Software is
# # furnished to do so, subject to the following conditions:
# #
# # The above copyright notice and this permission notice shall be included in
# # all copies or substantial portions of the Software.
# #
# # THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# # IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# # FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
# # AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# # LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# # THE SOFTWARE.

require_relative '../shared'

module GoodData
  module GoodotCLI
    desc 'Manage masters'
    command :masters do |c|
      c.desc 'List masters'
      c.command :list do |add|
        add.action do |global_options, _options, _args|
          domain = global_options[:gd_domain]
          currently_used = domain.segments.pmap { |s| [s, s.master_project] }.compact

          puts table_with_headers(currently_used,
                                  [
                                    proc { |_, p| p && p.pid },
                                    proc { |_, p| p && p.title },
                                    proc { |s, _| s.id },
                                    proc { |_, p| p && p.metadata['GD_LCM_VERSION'] }
                                  ],
                                  headers: ['PID', 'Title', 'Segment ID', 'Version'])
        end
      end

      c.desc 'Clone master of particular segment'
      c.command :clone do |add|
        add.desc 'Segment which master will be cloned'
        add.default_value nil
        add.flag [:s, :segment]

        add.desc 'Authorization token'
        add.default_value nil
        add.flag [:t, :token]

        add.action do |global_options, options, _args|
          domain = global_options[:gd_domain]
          fail 'You have to specify segment' unless options[:segment]
          master_project = domain.segments(options[:segment]).master_project
          clone = GoodData::Project.clone_with_etl(master_project, auth_token: options[:token])
          puts "Project \"#{master_project.title}\" (#{master_project.pid}) was cloned."
          puts "\"#{clone.title}\" (#{clone.pid}) was created."
        end
      end
    end
  end
end
