module GoodData
  module GoodotCLI
    desc 'Manage segments'
    command :segments do |c|
      c.desc 'List segments'
      c.command :list do |add|
        add.desc 'Format'
        add.default_value :table
        add.flag [:f, :format]

        add.action do |global_options, options, _args|
          domain = global_options[:gd_domain]

          segments = domain.segments
          case options[:format].to_sym
          when :table
            puts table_with_headers(segments, [:id, :master_project_id])
          when :json
            output = segments.map do |s|
              {
                name: s.id,
                master_project_id: s.master_project_id
              }
            end
            puts MultiJson.dump(output, pretty: true)
          end
        end
      end

      c.desc 'Remove segment'
      c.command [:remove, :delete] do |add|
        add.desc 'Segment'
        add.flag [:s, :segment]

        add.desc 'Delete also the clients in that particular segment'
        add.default_value false
        add.switch [:c, :cascade]

        add.action do |global_options, options, _args|
          domain = global_options[:gd_domain]
          segment_name = options[:segment]
          fail 'Segment needs to be provided' unless segment_name
          segment = domain.segments(segment_name)
          segment.clients.peach(&:delete) if options[:cascade]
          segment.delete
        end
      end

      c.desc 'Exchange master on segment.'
      c.command :exchange do |exchange|
        exchange.desc 'Segment'
        exchange.flag [:s, :segment]

        exchange.desc 'Master project ID'
        exchange.flag [:p, :project]

        exchange.desc 'Delete the old project'
        exchange.default_value false
        exchange.switch [:d, :delete]

        exchange.action do |global_options, options, _args|
          client = global_options[:gd_client]
          domain = global_options[:gd_domain]

          segment_name = options[:segment]
          fail 'Segment needs to be provided' unless segment_name
          segment = domain.segments(segment_name)

          project_id = options[:project]
          fail 'Project Master id needs to be provided' unless project_id
          old_master = segment.master_project
          new_master_project = client.projects(project_id)

          segment.master_project = new_master_project
          segment.save
          old_master.delete if options[:delete]
        end
      end
    end
  end
end
