# encoding: UTF-8
#
# Copyright (c) 2010-2016 GoodData Corporation. All rights reserved.
# This source code is licensed under the BSD-style license found in the
# LICENSE file in the root directory of this source tree.

require 'gli'
require 'pathname'

require_relative '../version'

module GoodData
  module GoodotCLI
    extend GLI::App

    desc 'GoodData user name'
    default_value nil
    arg_name 'gooddata-login'
    flag [:U, :username, :login]

    desc 'GoodData password'
    default_value nil
    arg_name 'gooddata-password'
    flag [:P, :password]

    desc 'Project pid'
    default_value nil
    arg_name 'project-id'
    flag [:p, :project_id]

    desc 'Server'
    default_value nil
    arg_name 'server'
    flag [:s, :server]

    desc 'Domain'
    default_value nil
    arg_name 'domain'
    flag [:d, :domain]

    desc 'Verbose mode'
    arg_name 'verbose'
    switch [:v, :verbose]

    desc 'Http logger on stdout'
    arg_name 'http-logger'
    default_value false
    switch [:l, 'http-logger']

    desc 'Http logger on stdout'
    arg_name 'logger'
    default_value true
    switch ['verify-ssl']

    pre do |global_options, _command, _options, _args|
      require 'gooddata'
      GoodData.logging_http_on if global_options[:'http-logger']
      client = GoodData.connect(login: global_options[:login],
                                password: global_options[:password],
                                server: global_options[:server],
                                verify_ssl: global_options['verify-ssl'])
      domain = client.domain(global_options[:domain])
      global_options[:gd_client] = client
      global_options[:gd_domain] = domain
      true
    end

    def self.create_api_error_message(e)
      HighLine.color(GoodData::Helpers.interpolate_error_message(MultiJson.load(e.http_body)), :red)
    end

    def self.table_with_headers(data, methods, options = {})
      limit = options[:limit] || data.size
      headers = options[:headers]
      Terminal::Table.new do |t|
        t << if headers
               headers
             else
               methods.map(&:to_s).map(&:capitalize)
             end
        t << :separator
        data.take(limit).each do |p|
          res = if p.is_a?(Hash)
                  methods.map { |m| m.is_a?(Proc) ? m.call(p) : p[m] }
                else
                  methods.map { |m| m.is_a?(Proc) ? m.call(p) : p.send(m) }
                end
          t.add_row(res)
        end
      end
    end
  end
end
